=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2011-02-14 16:51:05 +0000
+++ CMakeLists.txt	2011-02-17 17:19:57 +0000
@@ -12,7 +12,7 @@
 set (PROJECT_NAME "unity")
 set (UNITY_MAJOR 3)
 set (UNITY_MINOR 4)
-set (UNITY_MICRO 2)
+set (UNITY_MICRO 4)
 set (UNITY_VERSION "${UNITY_MAJOR}.${UNITY_MINOR}.${UNITY_MICRO}")
 set (UNITY_API_VERSION "3.0")
 
@@ -85,6 +85,12 @@
     SET (BOOT_LOGGER_FLAG "-DENABLE_LOGGER")
 endif (BOOT_LOGGER)
 
+SET (MAINTAINER_CFLAGS "-Werror -Wall -Wcast-align -Wno-uninitialized -Wempty-body -Wformat-security -Winit-self")
+option (DISABLE_MAINTAINER_CFLAGS "Disable maintainer CFlags" OFF)
+if (DISABLE_MAINTAINER_CFLAGS)
+  SET (MAINTAINER_CFLAGS "")
+endif (DISABLE_MAINTAINER_CFLAGS)
+
 #
 # src (Compiz Plugin)
 #
@@ -95,7 +101,7 @@
 compiz_plugin (unityshell
                PKGDEPS ${UNITY_PLUGIN_DEPS}
                PLUGINDEPS composite opengl
-               CFLAGSADD "-DINSTALLPREFIX='\"${CMAKE_INSTALL_PREFIX}\"' -DPKGDATADIR='\"${CMAKE_INSTALL_PREFIX}/share/unity/3\"' -I${CMAKE_BINARY_DIR} ${BOOT_LOGGER_FLAG} -DGETTEXT_PACKAGE='\"unity\"'"
+               CFLAGSADD "-DINSTALLPREFIX='\"${CMAKE_INSTALL_PREFIX}\"' -DPKGDATADIR='\"${CMAKE_INSTALL_PREFIX}/share/unity/3\"' -I${CMAKE_BINARY_DIR} ${BOOT_LOGGER_FLAG} -DGETTEXT_PACKAGE='\"unity\"' ${MAINTAINER_CFLAGS}"
                )
  
 #

=== modified file 'services/CMakeLists.txt'
--- services/CMakeLists.txt	2011-02-08 11:50:18 +0000
+++ services/CMakeLists.txt	2011-02-17 13:03:25 +0000
@@ -10,6 +10,7 @@
 set(CFLAGS
     ${SERVICE_DEPS_CFLAGS}
     ${SERVICE_DEPS_CFLAGS_OTHER}
+    ${MAINTAINER_CFLAGS}
     "-DGETTEXT_PACKAGE=\"unity\""
     "-DINDICATORDIR=\"${_indicatordir}\""
     "-DINDICATORICONDIR=\"${_iconsdir}\""

=== modified file 'src/Launcher.cpp'
--- src/Launcher.cpp	2011-02-21 11:34:16 +0000
+++ src/Launcher.cpp	2011-02-21 13:21:49 +0000
@@ -55,11 +55,24 @@
 
 int
 TimeDelta (struct timespec const *x, struct timespec const *y)
-
 {
   return ((x->tv_sec - y->tv_sec) * 1000) + ((x->tv_nsec - y->tv_nsec) / 1000000);
 }
 
+void SetTimeBack (struct timespec *timeref, int remove)
+{
+  timeref->tv_sec -= remove / 1000;
+  remove = remove % 1000;
+
+  if (remove > timeref->tv_nsec / 1000000)
+  {
+      timeref->tv_sec--;
+      timeref->tv_nsec += 1000000000;
+  }
+  timeref->tv_nsec -= remove * 1000000;
+}
+
+
 /*
 	        Use this shader to pass vertices in screen coordinates in the C++ code and compute use
 	        the fragment shader to perform the texture perspective correct division.
@@ -447,17 +460,19 @@
 float Launcher::AutohideProgress (struct timespec const &current)
 {
     
-    // bfb position progress
-    if (_mouse_inside_trigger)
+    // bfb position progress. Go from GetAutohidePositionMin() -> GetAutohidePositionMax() linearly
+    if (_mouse_inside_trigger && !_mouseover_launcher_locked)
     {
-        /* 
+        
+        // "dead" zone
+        if ((_trigger_mouse_position.x < 2) && (_trigger_mouse_position.y < 2))
+            return GetAutohidePositionMin ();
+        
+       /* 
         * most of the mouse movement should be done by the inferior part
         * of the launcher, so prioritize this one
         */
-        
-        if(_mouseover_launcher_locked || ((_trigger_mouse_position.x == 0) && (_trigger_mouse_position.y == 0)))
-            return 0.0f;
-        
+                
         float _max_size_on_position;
         float position_on_border = _trigger_mouse_position.x * _trigger_height / _trigger_mouse_position.y;
         
@@ -469,13 +484,14 @@
             _max_size_on_position = pow(pow(position_on_border, 2) + pow(_trigger_width, 2), 0.5);
         }
         // only triggered on _hidden = false, no need for check
-        return CLAMP (pow(pow(_trigger_mouse_position.x, 2) + pow(_trigger_mouse_position.y, 2), 0.5) / _max_size_on_position, 0.0f, 1.0f);  
+        float _position_min = GetAutohidePositionMin ();
+        return pow(pow(_trigger_mouse_position.x, 2) + pow(_trigger_mouse_position.y, 2), 0.5) / _max_size_on_position * (GetAutohidePositionMax () - _position_min) + _position_min;
     }
     
-    // time-based progress
+    // time-based progress (full scale or finish the TRIGGER_AUTOHIDE_MIN -> 0.00f on bfb)
     else
     {
-        float animation_progress;    
+        float animation_progress;
         animation_progress = CLAMP ((float) (TimeDelta (&current, &_times[TIME_AUTOHIDE])) / (float) ANIM_DURATION_SHORT, 0.0f, 1.0f);
         if (_hidden)
             return animation_progress;
@@ -618,21 +634,30 @@
         if (diff < sister_relation)
         {
             int remove = sister_relation - diff;
-            current.tv_sec -= remove / 1000;
-            remove = remove % 1000;
-
-            if (remove > current.tv_nsec / 1000000)
-            {
-                current.tv_sec--;
-                current.tv_nsec += 1000000000;
-            }
-            current.tv_nsec -= remove * 1000000;
+            SetTimeBack (&current, remove);
         }
     }
 
     timer->tv_sec = current.tv_sec;
     timer->tv_nsec = current.tv_nsec;
 }
+/* Min is when you lock the trigger */
+float Launcher::GetAutohidePositionMin ()
+{
+    if (_autohide_animation == SLIDE_ONLY)
+        return 0.55f;
+    else
+        return 0.25f;
+}
+/* Max is the initial state */
+float Launcher::GetAutohidePositionMax ()
+{
+    if (_autohide_animation == SLIDE_ONLY)
+        return 1.00f;
+    else
+        return 0.75f;
+}
+
 
 float IconVisibleProgress (LauncherIcon *icon, struct timespec const &current)
 {
@@ -1070,18 +1095,7 @@
 
         if (_autohide_animation == FADE_ONLY
             || (_autohide_animation == FADE_SLIDE && _mouse_inside_trigger))
-        {
-            if (autohide_progress > 0.0f)
-            {    
-                if (_mouse_inside_trigger)
-                {
-                    if (!_mouseover_launcher_locked)
-                        *launcher_alpha = 1.0f - (autohide_progress / 1.5f);
-                }
-                else
-                    *launcher_alpha = 1.0f - autohide_progress;
-            }
-        }
+            *launcher_alpha = 1.0f - autohide_progress;
         else
         {
             if (autohide_progress > 0.0f)
@@ -1091,8 +1105,14 @@
         * as we can go back and for. We have to lock the launcher manually changing
         * the _hidden state then
         */
-        if (autohide_progress == 0.0f && _mouse_inside_trigger && !_mouseover_launcher_locked)
+        float _position_min = GetAutohidePositionMin ();
+        if (autohide_progress == _position_min && _mouse_inside_trigger && !_mouseover_launcher_locked)
+        {
             ForceHiddenState (false); // lock the launcher
+            _times[TIME_AUTOHIDE] = current;
+            SetTimeBack (&_times[TIME_AUTOHIDE], ANIM_DURATION_SHORT * _position_min);
+            SetTimeStruct (&_times[TIME_AUTOHIDE], &_times[TIME_AUTOHIDE], ANIM_DURATION_SHORT); // finish the animation 
+        }
     }
     
     float drag_hide_progress = DragHideProgress (current);
@@ -2380,11 +2400,22 @@
 }
 
 void
+Launcher::CheckSuperShortcutPressed (unsigned int key_sym,
+                                     unsigned long key_code,
+                                     unsigned long key_state)
+{
+  if (_super_show_launcher)
+    RecvKeyPressed (key_sym, key_code, key_state);
+}
+
+void
 Launcher::RecvKeyPressed (unsigned int  key_sym,
                           unsigned long key_code,
                           unsigned long key_state)
 {
 
+  LauncherModel::iterator it;
+  
   switch (key_sym)
   {
     // up (move selection up or go to global-menu if at top-most icon)
@@ -2420,13 +2451,10 @@
       if (key_state & NUX_STATE_SHIFT)
       {
         {
-          LauncherModel::iterator it;
-          int i;
-
           // open quicklist of currently selected icon
-          for (it = _model->begin (), i = 0; it != _model->end (); it++, i++)
-            if (i == _current_icon_index)
-              (*it)->OpenQuicklist (true);
+          it = _model->at (_current_icon_index);
+          if (it != (LauncherModel::iterator)NULL)
+            (*it)->OpenQuicklist (true);
         }
         leaveKeyNavMode ();
       }
@@ -2434,13 +2462,10 @@
 
     case NUX_VK_RIGHT:
       {
-        LauncherModel::iterator it;
-        int i;
-
         // open quicklist of currently selected icon
-        for (it = _model->begin (), i = 0; it != _model->end (); it++, i++)
-          if (i == _current_icon_index)
-            (*it)->OpenQuicklist (true);
+        it = _model->at (_current_icon_index);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->OpenQuicklist (true);
       }
       leaveKeyNavMode ();
     break;
@@ -2448,13 +2473,10 @@
     // <SPACE> (open a new instance)
     case NUX_VK_SPACE:
       {
-        LauncherModel::iterator it;
-        int i;
-
         // start currently selected icon
-        for (it = _model->begin (), i = 0; it != _model->end (); it++, i++)
-          if (i == _current_icon_index)
-            (*it)->OpenInstance ();
+        it = _model->at (_current_icon_index);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->OpenInstance ();
       }
       leaveKeyNavMode ();
       break;
@@ -2462,17 +2484,86 @@
     // <RETURN> (start/activate currently selected icon)
     case NUX_VK_ENTER:
       {
-        LauncherModel::iterator it;
-        int i;
-
         // start currently selected icon
-        for (it = _model->begin (), i = 0; it != _model->end (); it++, i++)
-          if (i == _current_icon_index)
-            (*it)->Activate ();
+        it = _model->at (_current_icon_index);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
       }
       leaveKeyNavMode ();
     break;
 
+    // Shortcuts to launch applications
+    case XK_0:
+      {
+        it = _model->at (9);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_1:
+      {
+        it = _model->at (0);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_2:
+      {
+        it = _model->at (1);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_3:
+      {
+        it = _model->at (2);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_4:
+      {
+        it = _model->at (3);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_5:
+      {
+        it = _model->at (4);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_6:
+      {
+        it = _model->at (5);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_7:
+      {
+        it = _model->at (6);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_8:
+      {
+        it = _model->at (7);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+    case XK_9:
+      {
+        it = _model->at (8);
+        if (it != (LauncherModel::iterator)NULL)
+          (*it)->Activate ();
+        break;
+      }
+      
     default:
     break;
   }

=== modified file 'src/Launcher.h'
--- src/Launcher.h	2011-02-21 11:34:16 +0000
+++ src/Launcher.h	2011-02-21 13:21:49 +0000
@@ -121,6 +121,8 @@
   void SetAutoHideAnimation (AutoHideAnimation animation);
   AutoHideAnimation GetAutoHideAnimation ();
   
+  void CheckSuperShortcutPressed (unsigned int key_sym, unsigned long key_code, unsigned long key_state);
+  
   nux::BaseWindow* GetParent () { return _parent; };
 
   static void SetTimeStruct (struct timespec *timer, struct timespec *sister = 0, int sister_relation = 0);
@@ -325,6 +327,9 @@
   void StartIconDrag (LauncherIcon *icon);
   void EndIconDrag ();
   void UpdateDragWindowPosition (int x, int y);
+  
+  float GetAutohidePositionMin ();
+  float GetAutohidePositionMax ();
 
   virtual void PreLayoutManagement();
   virtual long PostLayoutManagement(long LayoutResult);

=== modified file 'src/LauncherModel.cpp'
--- src/LauncherModel.cpp	2011-02-10 01:18:16 +0000
+++ src/LauncherModel.cpp	2011-02-18 18:24:57 +0000
@@ -276,6 +276,22 @@
   return _inner.end ();
 }
 
+LauncherModel::iterator 
+LauncherModel::at (int index)
+{
+  LauncherModel::iterator it;
+  int i;
+
+  // start currently selected icon
+  for (it = _inner.begin (), i = 0; it != _inner.end (); it++, i++)
+  {
+    if (i == index)
+      return it;
+  }
+      
+  return (LauncherModel::iterator)NULL;
+}
+      
 LauncherModel::reverse_iterator 
 LauncherModel::rbegin ()
 {

=== modified file 'src/LauncherModel.h'
--- src/LauncherModel.h	2011-02-10 01:18:16 +0000
+++ src/LauncherModel.h	2011-02-18 18:24:57 +0000
@@ -50,6 +50,7 @@
     
     iterator begin ();
     iterator end ();
+    iterator at (int index);
     reverse_iterator rbegin ();
     reverse_iterator rend ();
     

=== modified file 'src/PlaceFactoryFile.cpp'
--- src/PlaceFactoryFile.cpp	2011-01-31 14:53:06 +0000
+++ src/PlaceFactoryFile.cpp	2011-02-17 16:01:55 +0000
@@ -28,8 +28,8 @@
 PlaceFactoryFile::PlaceFactoryFile (const char *directory)
 {
   //FIXME: This is a temporary Alpha 2 fix
-  g_spawn_command_line_async ("killall unity-files-daemon", NULL);
-  g_spawn_command_line_async ("killall unity-applications-daemon", NULL);
+  g_spawn_command_line_sync ("killall unity-files-daemon", NULL, NULL, NULL, NULL);
+  g_spawn_command_line_sync ("killall unity-applications-daemon", NULL, NULL, NULL, NULL);
 
   /* Use the default lookup location */
   if (directory == NULL)

=== modified file 'src/PlacesController.cpp'
--- src/PlacesController.cpp	2011-02-15 06:34:13 +0000
+++ src/PlacesController.cpp	2011-02-21 13:21:49 +0000
@@ -48,7 +48,7 @@
   _window_layout = new nux::HLayout ();
   
   _window = new nux::BaseWindow ("Dash");
-  _window->SetBackgroundColor (nux::Color (0.0, 0.0, 0.0, 0.7));
+  _window->SetBackgroundColor (nux::Color (0.0, 0.0, 0.0, 0.85));
   _window->SinkReference ();
   _window->SetConfigureNotifyCallback(&PlacesController::WindowConfigureCallback, this);
   _window->ShowWindow(false);

=== modified file 'src/PlacesResultsController.cpp'
--- src/PlacesResultsController.cpp	2011-02-15 13:22:40 +0000
+++ src/PlacesResultsController.cpp	2011-02-17 16:48:20 +0000
@@ -81,6 +81,10 @@
   {
     group->SetVisible (true);
     _results_view->ReJiggyGroups ();
+
+    group->QueueDraw ();
+    group->ComputeChildLayout ();
+    group->GetLayout ()->QueueDraw ();
   }
 }
 
@@ -102,6 +106,12 @@
         group->SetVisible (false);
         _results_view->ReJiggyGroups ();
       }
+      else
+      {
+        group->QueueDraw ();
+        group->GetLayout ()->QueueDraw ();
+        group->ComputeChildLayout ();
+      }
     }
     else
     {

=== modified file 'src/PlacesResultsView.cpp'
--- src/PlacesResultsView.cpp	2011-02-10 14:34:30 +0000
+++ src/PlacesResultsView.cpp	2011-02-17 16:01:55 +0000
@@ -57,6 +57,9 @@
     if ((*it)->IsVisible ())
     {
       _layout->AddView ((*it), 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
+      _layout->QueueDraw ();
+      (*it)->QueueDraw ();
+      QueueDraw ();
     }
   }
 }

=== modified file 'src/PlacesView.cpp'
--- src/PlacesView.cpp	2011-02-10 16:34:36 +0000
+++ src/PlacesView.cpp	2011-02-17 16:31:42 +0000
@@ -88,7 +88,23 @@
 long
 PlacesView::ProcessEvent(nux::IEvent &ievent, long TraverseInfo, long ProcessEventInfo)
 {
+  // FIXME: This breaks with multi-monitor
+  nux::Geometry homebutton (0.0f, 0.0f, 66.0f, 24.0f);
   long ret = TraverseInfo;
+
+  if (ievent.e_event == nux::NUX_KEYDOWN
+      && ievent.GetKeySym () == NUX_VK_ESCAPE)
+  {
+    SetActiveEntry (NULL, 0, "");
+    return TraverseInfo;
+  }
+
+  if (ievent.e_event == nux::NUX_MOUSE_RELEASED)
+  {
+    if (homebutton.IsPointInside (ievent.e_x, ievent.e_y))
+      SetActiveEntry (NULL, 0, "");
+    return TraverseInfo;
+  }
     
   ret = _layout->ProcessEvent (ievent, ret, ProcessEventInfo);
   return ret;

=== modified file 'src/unityshell.cpp'
--- src/unityshell.cpp	2011-02-21 11:34:16 +0000
+++ src/unityshell.cpp	2011-02-21 13:21:49 +0000
@@ -188,7 +188,11 @@
         PluginAdapter::Default ()->OnScreenGrabbed ();
       else if (event->xfocus.mode == NotifyUngrab)
         PluginAdapter::Default ()->OnScreenUngrabbed ();
-
+      break;
+    case KeyPress:
+      KeySym key_sym;
+      if (XLookupString(&(event->xkey), NULL, 0, &key_sym, 0) > 0)
+          launcher->CheckSuperShortcutPressed (key_sym, 0, 0);
       break;
   }
 

=== modified file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2011-02-17 15:54:54 +0000
+++ tests/CMakeLists.txt	2011-02-21 13:21:49 +0000
@@ -14,6 +14,7 @@
 set (CFLAGS
      ${TEST_UNIT_DEPS_CFLAGS}
      ${TEST_UNIT_DEPS_CFLAGS_OTHER}
+     ${MAINTAINER_CFLAGS}
      "-DTESTDATADIR=${TESTDATADIR}"
      "-DGETTEXT_PACKAGE=\"unity\""
      "-DINDICATORDIR=\"${CMAKE_BINARY_DIR}/tests\""

=== modified file 'tests/TestPlacesGroup.cpp'
--- tests/TestPlacesGroup.cpp	2011-02-07 11:43:30 +0000
+++ tests/TestPlacesGroup.cpp	2011-02-17 13:03:25 +0000
@@ -134,8 +134,9 @@
   nux::SleepForMilliseconds (3000);
   printf ("ControlThread successfully started\n");
 
-  nux::WindowThread* mainWindowThread = NUX_STATIC_CAST (nux::WindowThread*,
-                                                         data);
+  nux::WindowThread* mainWindowThread;
+  
+  mainWindowThread = NUX_STATIC_CAST (nux::WindowThread*, data);
 }
 
 

=== modified file 'tests/TestPlacesResults.cpp'
--- tests/TestPlacesResults.cpp	2011-02-10 16:34:36 +0000
+++ tests/TestPlacesResults.cpp	2011-02-17 13:03:25 +0000
@@ -135,8 +135,9 @@
   nux::SleepForMilliseconds (3000);
   printf ("ControlThread successfully started\n");
 
-  nux::WindowThread* mainWindowThread = NUX_STATIC_CAST (nux::WindowThread*,
-                                                         data);
+  nux::WindowThread* mainWindowThread;
+  
+  mainWindowThread = NUX_STATIC_CAST (nux::WindowThread*, data);
 }
 
 

=== modified file 'tests/TestPlacesTiles.cpp'
--- tests/TestPlacesTiles.cpp	2011-01-19 17:08:14 +0000
+++ tests/TestPlacesTiles.cpp	2011-02-17 13:03:25 +0000
@@ -83,8 +83,9 @@
   nux::SleepForMilliseconds (3000);
   printf ("ControlThread successfully started\n");
 
-  nux::WindowThread* mainWindowThread = NUX_STATIC_CAST (nux::WindowThread*,
-                                                         data);
+  nux::WindowThread* mainWindowThread;
+  
+  mainWindowThread = NUX_STATIC_CAST (nux::WindowThread*, data);
 }
 
 

